diff -ru nginx-0.5.35/src/http/modules/ngx_http_realip_module.c nginx-0.5.35-patched/src/http/modules/ngx_http_realip_module.c
--- nginx-0.5.35/src/http/modules/ngx_http_realip_module.c	2007-09-22 12:02:39.000000000 -0700
+++ nginx-0.5.35-patched/src/http/modules/ngx_http_realip_module.c	2008-01-18 12:10:18.000000000 -0800
@@ -9,6 +9,10 @@
 #include <ngx_http.h>
 
 
+#define NGX_HTTP_XFWD_X_REAL_IP           -2
+#define NGX_HTTP_XFWD_X_FORWARDED_FOR     -3
+
+
 /* AF_INET only */
 
 typedef struct {
@@ -20,26 +24,21 @@
 typedef struct {
     ngx_array_t  *from;     /* array of ngx_http_realip_from_t */
 
-    ngx_uint_t    xfwd;
+    ngx_int_t     xfwd;
 } ngx_http_realip_loc_conf_t;
 
 
 static ngx_int_t ngx_http_realip_handler(ngx_http_request_t *r);
 static char *ngx_http_realip_from(ngx_conf_t *cf, ngx_command_t *cmd,
     void *conf);
+static char *ngx_http_realip_set_header_slot(ngx_conf_t *cf,
+    ngx_command_t *cmd, void *conf);
 static void *ngx_http_realip_create_loc_conf(ngx_conf_t *cf);
 static char *ngx_http_realip_merge_loc_conf(ngx_conf_t *cf,
     void *parent, void *child);
 static ngx_int_t ngx_http_realip_init(ngx_conf_t *cf);
 
 
-static ngx_conf_enum_t  ngx_http_realip_header[] = {
-    { ngx_string("X-Forwarded-For"), 1 },
-    { ngx_string("X-Real-IP"), 0 },
-    { ngx_null_string, 0 }
-};
-
-
 static ngx_command_t  ngx_http_realip_commands[] = {
 
     { ngx_string("set_real_ip_from"),
@@ -51,10 +50,10 @@
 
     { ngx_string("real_ip_header"),
       NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
-      ngx_conf_set_enum_slot,
+      ngx_http_realip_set_header_slot,
       NGX_HTTP_LOC_CONF_OFFSET,
       offsetof(ngx_http_realip_loc_conf_t, xfwd),
-      &ngx_http_realip_header },
+      NULL },
 
       ngx_null_command
 };
@@ -102,6 +101,7 @@
     struct sockaddr_in          *sin;
     ngx_http_realip_from_t      *from;
     ngx_http_realip_loc_conf_t  *rlcf;
+    ngx_http_variable_value_t   *vv;
 
     if (r->realip_set) {
         return NGX_DECLINED;
@@ -113,15 +113,19 @@
         return NGX_DECLINED;
     }
 
-    if (rlcf->xfwd == 0) {
+    switch (rlcf->xfwd) {
+    case NGX_HTTP_XFWD_X_REAL_IP:
+
         if (r->headers_in.x_real_ip == NULL) {
             return NGX_DECLINED;
         }
 
         len = r->headers_in.x_real_ip->value.len;
         ip = r->headers_in.x_real_ip->value.data;
+	break;
+
+    case NGX_HTTP_XFWD_X_FORWARDED_FOR:
 
-    } else {
         if (r->headers_in.x_forwarded_for == NULL) {
             return NGX_DECLINED;
         }
@@ -137,6 +141,18 @@
                 break;
             }
         }
+        break;
+
+    default:
+
+        vv = ngx_http_get_indexed_variable(r, rlcf->xfwd);
+
+        if (vv == NULL || vv->not_found) {
+            return NGX_ERROR;
+        }
+
+        len = vv->len;
+        ip = vv->data;
     }
 
     ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
@@ -236,6 +252,48 @@
 }
 
 
+static char *
+ngx_http_realip_set_header_slot(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
+{
+    char  *p = conf;
+
+    ngx_int_t                     *cp;
+    ngx_str_t                     *value, var;
+
+    cp = (ngx_int_t *) (p + cmd->offset);
+
+    if (*cp != NGX_CONF_UNSET) {
+        return "is duplicate";
+    }
+
+    value = cf->args->elts;
+
+    if (ngx_strcmp(value[1].data, "X-Real-IP") == 0)
+    {
+        *cp = NGX_HTTP_XFWD_X_REAL_IP;
+    }
+    else if (ngx_strcmp(value[1].data, "X-Forwarded-For") == 0)
+    {
+        *cp = NGX_HTTP_XFWD_X_FORWARDED_FOR;
+    }
+    else if (value[1].data[0] == '$') {
+        var.len = value[1].len - 1;
+        var.data = value[1].data + 1;
+
+        *cp = ngx_http_get_variable_index(cf, &var);
+
+        if (*cp == NGX_ERROR) {
+            return NGX_CONF_ERROR;
+        }
+    }
+    else {
+        return NGX_CONF_ERROR;
+    }
+
+    return NGX_CONF_OK;
+}
+
+
 static void *
 ngx_http_realip_create_loc_conf(ngx_conf_t *cf)
 {
@@ -252,7 +310,7 @@
      *     conf->from = NULL;
      */
 
-    conf->xfwd = NGX_CONF_UNSET_UINT;
+    conf->xfwd = NGX_CONF_UNSET;
 
     return conf;
 }
@@ -268,7 +326,7 @@
         conf->from = prev->from;
     }
 
-    ngx_conf_merge_uint_value(conf->xfwd, prev->xfwd, 0);
+    ngx_conf_merge_value(conf->xfwd, prev->xfwd, NGX_HTTP_XFWD_X_REAL_IP);
 
     return NGX_CONF_OK;
 }